#############################################################
#
#   Program: outlrm_model1.R
#   Project: 
# 
#   PI: 
#   Statistician: Bryan Shephard, PhD
#   Biostatistician/Programmer: Meridith Blevins, MS
#   Purpose: 
#   Created: 29 February 2012
#   Revisions: 
#     
#
#############################################################

set.seed(1)
library(rms)

nicu$birth_kg <- nicu$birth_weight/1000
nicu$nicu_admit_RDS <- nicu$nicu_admit_reason___2
nicu$nicu_admit_sepsis <- nicu$nicu_admit_reason___1
nicu$death <- abs(nicu$nicu_death-2)
nicu$prom <- ifelse(nicu$prom.factor=="Yes",1,0); nicu$prom[nicu$prom.factor=="Unknown or no data available"] <- NA
nicu$preeclampsia <- ifelse(nicu$maternal_preeclampsia.factor=="Yes",1,0); nicu$preeclampsia[nicu$maternal_preeclampsia.factor=="Unknown or no data available"] <- NA
nicu$gestational_age <- forcenumber(nicu$gestational_age)
nicu$vent <- ifelse(nicu$mechanical_vent.factor=="Yes",1,0); nicu$vent[nicu$mechanical_vent.factor=="Unknown or no data available"] <- NA
nicu$hood <- ifelse(nicu$oxyhood.factor=="Yes",1,0); nicu$hood[nicu$oxyhood.factor=="Unknown or no data available"] <- NA
nicu$canula <- ifelse(nicu$nasal_canula.factor=="Yes",1,0); nicu$canula[nicu$nasal_canula.factor=="Unknown or no data available"] <- NA

lrm1.d1 <- with(nicu,data.frame(death,sex.factor,gestational_age,nicu_admit_sepsis,nicu_admit_RDS,birth_kg,apgar_score_1_minute,apgar_score_5_minutes,vitamin_d_level,antib_number_days,prom,preeclampsia,crp_everpos.factor,type_of_delivery.factor,vent,hood,canula))
lrm1.dd1 <- datadist(lrm1.d1)
options(datadist='lrm1.dd1')

y1 <- lrm(death~rcs(gestational_age,3)+sex.factor,data=lrm1.d1)
y2 <- lrm(death~rcs(gestational_age,3)+nicu_admit_sepsis,data=lrm1.d1)
y3 <- lrm(death~rcs(gestational_age,3)+nicu_admit_RDS,data=lrm1.d1)
y4 <- lrm(death~rcs(gestational_age,3)+rcs(birth_kg,3),data=lrm1.d1)
y5 <- lrm(death~rcs(gestational_age,3)+apgar_score_1_minute,data=lrm1.d1)
y6 <- lrm(death~rcs(gestational_age,3)+apgar_score_5_minutes,data=lrm1.d1)
#y7 <- lrm(death~rcs(gestational_age,3)+vitamin_d_level,data=lrm1.d1)
y8 <- lrm(death~rcs(gestational_age,3)+antib_number_days,data=lrm1.d1)
y9 <- lrm(death~rcs(gestational_age,3)+prom,data=lrm1.d1)
y10 <- lrm(death~rcs(gestational_age,3)+preeclampsia,data=lrm1.d1)
y11 <- lrm(death~rcs(gestational_age,3)+crp_everpos.factor,data=lrm1.d1)


y12 <- lrm(death~rcs(gestational_age,3)+type_of_delivery.factor,data=lrm1.d1)
y13 <- lrm(death~rcs(gestational_age,3)+vent,data=lrm1.d1)
y14 <- lrm(death~rcs(gestational_age,3)+hood,data=lrm1.d1)
y15 <- lrm(death~rcs(gestational_age,3)+canula,data=lrm1.d1)

y0 <- lrm(death~rcs(gestational_age,3),data=lrm1.d1)



getOR <- function(y,label,index=3){
  n <- sum(y$freq)
  lrmsum <- summary(y)[4,c(4,6,7)]
  pval <- myformat4(as.data.frame(anova(y))$P[index])
  lrmsumlines1 <- paste(label,n,paste(sapply(lrmsum[1],myformat2)," (",sapply(lrmsum[2],myformat2),", ",sapply(lrmsum[3],myformat2),")",sep=""),pval,sep=" & ")
  return(lrmsumlines1)
}
# getOR(y1,"Female")
# getOR(y2,"Admitted for sepsis")
# getOR(y3,"Admitted for RDS")
# getOR(y4,"Birth weight (3 vs. 2 kg)\\\\tnote{b}")
# getOR(y5,"Apgar at 1 minute (per 2 increase)")
# getOR(y6,"Apgar at 5 minutes (per 1 increase)")
# getOR(y7,"Vitamin D (per 2.6 increase)")
# getOR(y8,"Days on antib_number_days (per 5 days)")
# getOR(y9,"PROM")
# getOR(y10,"Preeclampsia")
# getOR(y11,"Positive CRP")


lrm1latex <- paste(getOR(y1,"Female")
		  ,getOR(y2,"Admitted for sepsis")
		  ,getOR(y3,"Admitted for RDS")
		  ,getOR(y4,"Birth weight (3 vs. 2 kg)\\\\tnote{b}")
		  ,getOR(y5,"Apgar at 1 minute (per 2 increase)")
		  ,getOR(y6,"Apgar at 5 minutes (per 1 increase)")
		  #,getOR(y7,"Vitamin D (per 2.6 increase)")
		  ,getOR(y8,"Days on antibiotics (per 5 days)")
		  ,getOR(y9,"PROM")
		  ,getOR(y10,"Preeclampsia")
		  ,getOR(y11,"Positive CRP")
		  ,getOR(y12,"C-section (vs. vaginal)")
		  ,getOR(y13,"Received mechanical ventalation")
		  ,getOR(y14,"Received oxyhood")
		  ,getOR(y15,"Received nasal canula"),sep=" \\\\\\\\ ")


myadj1 <- 1
myadj2 <- -0.5
png("graphics/death_logodds_age.png",res=300,width=1250,height=1050, bg="transparent") 
par(mfrow=c(1,1))
par(mgp=c(2,1,0))
p <- Predict(y0,gestational_age)
print(plot(p,ylab="Log odds of death",xlab="Gestational age (weeks)",adj.subtitle=FALSE,data=lrm1.d1,lwd=2,scat1d.opts=list(col="darkred",lwd=0.4,frac=0.05)     ,par.settings = list(
        axis.components = list(
           left   = list(pad1 = myadj1, pad2 = myadj2)
          ,bottom = list(pad1 = myadj1, pad2 = myadj2)
          ,top    = list(pad1 = myadj1, pad2 = myadj2)
          ,right  = list(pad1 = myadj1, pad2 = myadj2)
        )
)))
dev.off()


myadj1 <- 1
myadj2 <- -0.5
png("graphics/death_prob_age.png",res=300,width=1250,height=1050, bg="transparent") 
par(mfrow=c(1,1))
par(mgp=c(2,1,0))
p <- Predict(y0,gestational_age,fun=plogis)
print(plot(p,ylab="Probability of death",xlab="Gestational age (weeks)",adj.subtitle=FALSE,data=lrm1.d1,lwd=2,scat1d.opts=list(col="darkred",lwd=0.4,frac=0.05)     ,par.settings = list(
        axis.components = list(
           left   = list(pad1 = myadj1, pad2 = myadj2)
          ,bottom = list(pad1 = myadj1, pad2 = myadj2)
          ,top    = list(pad1 = myadj1, pad2 = myadj2)
          ,right  = list(pad1 = myadj1, pad2 = myadj2)
        )
)))
dev.off()
