## CHOOSE FIRST SELECTS THE TEXT STRING OCCURING BEFORE THE SPECIFIED SEPARATER
choosefirst <- function(var,sep=".",num=1) unlist(lapply(strsplit(var,sep,fixed=TRUE),function(x) x[num]))

myformat1 <- function(x){format(round(x,1),nsmall=1)}
myformat2 <- function(x){format(round(x,2),nsmall=2)}
myformat3 <- function(x){
    y <- NULL
    if(x < 0.01){y <- "<0.01"}
    else{y <- format(round(x,2),nsmall=2)}
    return(y)
}
myformat4 <- function(x){
    y <- NULL
    if(x < 0.001){y <- "<0.001"}
    else{y <- format(round(x,3),nsmall=3)}
    return(y)
}
myper0 <- function(x){
    y <- NULL
    if(x <= 0.5 & x!=0){y <- "<1"}
    else{y <- format(round(x),nsmall=0)}
    return(y)
}
myper1 <- function(x){
    y <- NULL
    if(x < 0.05 & x!=0){y <- "<0.1"}
    else{y <- format(round(x,1),nsmall=1)}
    return(y)
}
medianIQR <- function(var,digits=1){
  paste(format(round(median(var,na.rm=TRUE),digits),nsmall=digits)," (",format(round(quantile(var,0.25,na.rm=TRUE),digits),nsmall=digits)," -- ",format(round(quantile(var,0.75,na.rm=TRUE),digits),nsmall=digits),")",sep="")
}
meanSD <- function(var,digits=1){
  paste(format(round(mean(var,na.rm=TRUE),digits),nsmall=digits)," (",format(round(sd(var,na.rm=TRUE),digits),nsmall=digits),")",sep="")
}
getrange <- function(var,digits=1){
  summary <- range(var,na.rm=TRUE)
  paste(format(round(summary[1],digits),nsmall=digits)," -- ",format(round(summary[2],digits),nsmall=digits),sep="")
}
## THIS IS A WRAPPER TO LATEX ANY DATA FRAME FOR THE TABLE'S "GUTS"
latexdf <- function(df){
  paste(apply(df,1,paste,collapse=" & "),collapse=" \\\\\\\\ ")
  }
tablemiss <- function(var){
  table(var,useNA='always')
}
# returns string w/o leading or trailing whitespace
trim <- function (x) gsub("^\\s+|\\s+$", "", x)

## Check that a vector of items are all equal (used to check that column counts/names are the same in multiple datasets)
all.items.equal <- function(vector){
  count <- 0
  i <- length(vector)
  getcombn <- combn(i,2)
  j <- ncol(getcombn)
  for(k in 1:j){
    count <- count + !(vector[getcombn[1,k]]==vector[getcombn[2,k]])
  }
  ans <- ifelse(count>0,"STOP: Unequal items","Equal items")
  return(ans)
}


## FORCE NUMBER will take a factor/character to a number without issuing the warnings
forcenumber <- function(var){
    options(warn = -1)
    neg <- ifelse(substr(var,1,1)=="-","-","")
    x <- as.numeric(paste0(neg,gsub("[^0-9.]","",as.character(var))))
    options(warn = 1)
    return(x)
}

## PRINT DIAGNOSTIC ACCURACY
calcprop <- function(x,n){
 p <- binconf(x,n,alpha=0.05,method="wilson")
 if(length(p)==3)  p1 <- paste(myformat2(p[1])," (",myformat2(p[2]),", ",myformat2(p[3]),") [",x,"/",n,"]",sep="")
 if(length(p)>3)  p1 <- paste(myformat2(p[,1])," (",myformat2(p[,2]),", ",myformat2(p[,3]),") [",x,"/",n,"]",sep="")
 return(p1)
}
getacctex <- function(TP,TN,FP,FN,by,label,rowlabel,indent=TRUE){
  if(length(rowlabel) != sum(!(unique(by)%in% NA))){warning("Function GETACCTEX: number of unique values does not match length of rowlabel")}
  SN1 <- table(TP,by)[2,]
  SN2 <- table(TP | FN,by)[2,]
  SN <- calcprop(SN1,SN2)
  SP1 <- table(TN,by)[2,]
  SP2 <- table(FP | TN,by)[2,]
  SP <- calcprop(SP1,SP2)
  PPV1 <- table(TP,by)[2,]
  PPV2 <- table(TP | FP,by)[2,]
  PPV <- calcprop(PPV1,PPV2)
  NPV1 <- table(TN,by)[2,]
  NPV2 <- table(FN | TN,by)[2,]
  NPV <- calcprop(NPV1,NPV2)
  getn <- table(TP|TN|FP|FN,by)
  lines1 <- ifelse(indent,paste(paste(paste("\\\\hspace{.2in} ",rowlabel," (n=",getn,")",sep=""),SN,SP,PPV,NPV,sep=" & "),collapse=" \\\\\\\\ "),paste(paste(paste(rowlabel," (n=",getn,")",sep=""),SN,SP,PPV,NPV,sep=" & "),collapse=" \\\\\\\\ "))
  lines <- gsub("NaN","--",ifelse(!is.na(label),paste(c(label,lines1),collapse=" \\\\\\\\ "),lines1))
  return(lines)
}

## PRINT LATEX HEADER FOR GETNPTEX, GETMEDTEX, and GETNP1LINE functions
printheader <- function(crosstab,total=TRUE,total.label="Combined",pval=FALSE,pval.label="P-value"){
if(total){
  x <- paste(c("",names(crosstab),total.label),collapse=" & ")
  y <- paste(c("",paste("(n=",c(crosstab,sum(crosstab)),")",sep="")),collapse=" & ")
  }
if(!total){
  x <- paste(c("",names(crosstab)),collapse=" & ")
  y <- paste(c("",paste("(n=",c(crosstab),")",sep="")),collapse=" & ")
  }
if(pval){x <- paste(c(x,pval.label),collapse=" & ")}
  line <- paste(c(x,y),collapse=" \\\\\\\\ ")
  return(line)
}

getmeansdtex <- function(var,by,label,note=note,inc.pval=TRUE,lower.label=TRUE,miss.label=TRUE,by.order=FALSE,digits=8,formatper="myper0"){
  myper1 <- get(formatper)
  if(!is.numeric(var)){stop("The variable is non-numeric")}
  text <- meanSD(var)
  level <- sort(unique(by))
  textby <- rep(NA,length(level))
  for(i in 1:length(level)){
      textby[i] <- meanSD(var[by==level[i]])
  }
  if(inc.pval==TRUE){
      if(length(level)==1) {pval <- " "}
      else if(length(level)>1 & by.order==FALSE) {pval <- myformat4(kruskal.test(var ~ by)$p.value)}
      #else if(length(level)>1 & by.trend==TRUE)  {require(clinfun); pval <- myformat4(jonckheere.test(var,as.ordered(by),alternative="two.sided")$p.value)}
      else if(length(level)>1 & by.order==TRUE)  {pval <- myformat4(cor.test(var, as.numeric(by),alternative="two.sided",method="spearman")$p.value)}
      if(missing(note)){line1 <- paste(label,paste(textby,collapse=' & '),text,pval,sep=' & ')
      } else{line1 <- paste(paste(label,"\\\\tnote{",note,"}",sep=""),paste(textby,collapse=' & '),text,pval,sep=' & ')
      }
  } else{
      if(missing(note)){line1 <- paste(label,paste(textby,collapse=' & '),text,sep=' & ')
      } else{line1 <- paste(paste(label,"\\\\tnote{",note,"}",sep=""),paste(textby,collapse=' & '),text,sep=' & ')
      }
  }
  if(sum(is.na(var))>0){
    no <- table(is.na(var),by)[2,]
    nop <- sapply(100*no/table(by),myper1)
    nopt <- sapply(100*sum(no)/sum(table(by)),myper1)
    if(lower.label==TRUE){line2 <- paste(paste("\\\\hspace{.1in}Missing",ifelse(miss.label,paste(" ",tolower(label),sep=""),""),", n(\\\\%)",sep=""),paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),paste(sum(no)," (",nopt,"\\\\%)",sep=""),sep=" & ")}
      else{line2 <- paste(paste("\\\\hspace{.1in}Missing",ifelse(miss.label,paste(" ",label,sep=""),""),", n(\\\\%)",sep=""),paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),paste(sum(no)," (",nopt,"\\\\%)",sep=""),sep=" & ")}
    line3 <- paste(line1,line2,sep=" \\\\\\\\ ")
    return(list(line=line1,line.incmiss=line3))
  }
  else{return(list(line=line1))}
}


## PRINT ONE LINE FOR VARIABLES WITH TWO CATEGORIES OR ONLY ONE OF INTEREST 
## EXAMPLE (PRINT FEMALE (%) -- NO NEED TO PRINT MALE(%) AS WELL)
# var must be a variable that takes on 1 value and the rest are NA
# similarly, rowlabel must be length 1
getnp1line <- function(var,by,rowlabel="",indent=TRUE,inc.pval=FALSE,by.order=FALSE,formatper="myper0",rowper=FALSE,noper=FALSE){
  rowlabel <- ifelse(rowlabel=="",label(var),rowlabel)
  if(rowlabel==""){stop("rowlabel was not specified, either as an attribute of var, or in the getnp1line call.")}
  myper1 <- get(formatper)
  n <- table(var,by)
  d <- table(by)
  n1 <- table(var)
  if(!rowper){p <- sapply(100*n[1,]/d,myper1)}
  if(rowper){p <- sapply(100*n[1,]/n1,myper1)}
  pt <- sapply(100*n1/length(by),myper1)
  if(indent==TRUE){line <- paste(paste("\\\\hspace{.2in}",rowlabel),paste(paste(n[1,]," (",p,"\\\\%)",sep=""),collapse=" & "),paste(sum(n)," (",pt,"\\\\%)",sep=""),sep=" & ")}
  else{line <- paste(paste(rowlabel),paste(paste(n[1,]," (",p,"\\\\%)",sep=""),collapse=" & "),paste(sum(n)," (",pt,"\\\\%)",sep=""),sep=" & ")}
  if(indent & noper){line <- paste(paste("\\\\hspace{.2in}",rowlabel),paste(n[1,],collapse=" & "),sum(n),sep=" & ")}
  if(!indent & noper){line <- paste(paste(rowlabel),paste(n[1,],collapse=" & "),sum(n),sep=" & ")}
  if(inc.pval==TRUE){
    ##if(sum(is.na(var) & is.na(by))/length(var) < .9) warning("Function NP1LINE: assuming if var!=1 then var=0 for hypothesis test; NOTE: more than 10% of data is missing")
    var1 <- ifelse(is.na(var),0,var)
    if(by.order==TRUE){pval <- myformat4(wilcox.test(as.numeric(by)~ var1)$p.value)}
    if(by.order==FALSE){pval <- myformat4(chisq.test(table(var1,by))$p.value)}
    line <- paste(line,pval,sep=" & ")
    }
  return(list(line=line))
}

getnp1linetex <- function(var,by,rowlabel="",indent=TRUE,inc.pval=FALSE,by.order=FALSE,lower.label=TRUE,miss.label=FALSE,formatper="myper0",rowper=FALSE,noper=FALSE){
  rowlabel <- ifelse(rowlabel=="",label(var),rowlabel)
  if(rowlabel==""){stop("rowlabel was not specified, either as an attribute of var, or in the getnp1line call.")}
  var <- as.numeric(var)
  if(!all(sapply(var,function(var)var==0|var==1|is.na(var)))){stop("var must equal TRUE, FALSE, 1, 0, or missing")}
  myper1 <- get(formatper)
  if(dim(table(var))!=2){stop("var must have 2 unique values equal to TRUE/FALSE or 1/0")}
  n <- table(var,by)[2,]
  d <- table(by[!is.na(var)])
  n1 <- table(var)[2]
  if(!rowper){p <- sapply(100*n/d,myper1)}
  if(rowper){p <- sapply(100*n/n1,myper1)}
  pt <- sapply(100*n1/length(by),myper1)
  if(indent==TRUE){line <- paste(paste("\\\\hspace{.2in}",rowlabel),paste(paste(n," (",p,"\\\\%)",sep=""),collapse=" & "),paste(sum(n)," (",pt,"\\\\%)",sep=""),sep=" & ")}
  else{line <- paste(paste(rowlabel),paste(paste(n," (",p,"\\\\%)",sep=""),collapse=" & "),paste(sum(n)," (",pt,"\\\\%)",sep=""),sep=" & ")}
  if(indent & noper){line <- paste(paste("\\\\hspace{.2in}",rowlabel),paste(n,collapse=" & "),sum(n),sep=" & ")}
  if(!indent & noper){line <- paste(paste(rowlabel),paste(n,collapse=" & "),sum(n),sep=" & ")}
  if(inc.pval==TRUE){
    ##if(sum(is.na(var) & is.na(by))/length(var) < .9) warning("Function NP1LINE: assuming if var!=1 then var=0 for hypothesis test; NOTE: more than 10% of data is missing")
    var1 <- ifelse(is.na(var),0,var)
    if(by.order==TRUE){pval <- myformat4(wilcox.test(as.numeric(by)~ var1)$p.value)}
    if(by.order==FALSE){pval <- myformat4(chisq.test(table(var1,by))$p.value)}
    line <- paste(line,pval,sep=" & ")
    }
  if(sum(is.na(var))==0){
     return(list(line=line))
   }
  if(sum(is.na(var))>0){
    no <- table(is.na(var),by)[2,]
    nop <- sapply(100*no/table(by),myper1)
    nopt <- sapply(100*sum(no)/sum(table(by)),myper1)
    if(lower.label==TRUE){line2 <- paste(paste(ifelse(indent,"\\\\hspace{.3in}Missing","\\\\hspace{.1in}Missing"),ifelse(miss.label,paste(" ",tolower(rowlabel),sep=""),""),", n(\\\\%)",sep=""),paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),paste(sum(no)," (",nopt,"\\\\%)",sep=""),sep=" & ")}
      else{line2 <- paste(paste(ifelse(indent,"\\\\hspace{.3in}Missing","\\\\hspace{.1in}Missing"),ifelse(miss.label,paste(" ",rowlabel,sep=""),""),", n(\\\\%)",sep=""),paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),paste(sum(no)," (",nopt,"\\\\%)",sep=""),sep=" & ")}
    line1 <- paste(line,line2,sep=" \\\\\\\\ ")
    return(list(line=line,line.incmiss=line1))
  }
}
## PRINT MED (IQR) FOR VAR WITHIN EACH BY COLUMN AND OVERALL
# options include 'by.order'' which will affect the hypothesis test
# note will include \tnote{note} by the label
# OUTPUT INCLUDES line and line.incmiss (if there is missing data)
getmedtex <- function(var,by,label="",note=note,inc.pval=TRUE,lower.label=TRUE,miss.label=TRUE,by.order=FALSE,digits=8,formatper="myper0",iqrsep=" -- "){
  label <- ifelse(label=="",label(var),label)
  if(label==""){stop("label was not specified, either as an attribute of var, or in the getmedtex call.")}
  myper1 <- get(formatper)
  if(!is.numeric(var)){stop("The variable is non-numeric")}
  s <- round(summary(var),digits=digits)
  text <- paste(s[3]," (",s[2],iqrsep,s[5],")",sep="")
  level <- sort(unique(by))
  sby <- matrix(nrow=length(level), ncol=length(s)); textby <- rep(NA,length(level))
  for(i in 1:length(level)){
      sby[i,1:5] <- round(summary(var[by==level[i]])[1:5],digits=digits)
      textby[i] <- ifelse(!is.na(sby[i,3]),paste(sby[i,3]," (",sby[i,2],iqrsep,sby[i,5],")",sep=""), "--")
  }
  if(inc.pval==TRUE){
      if(length(level)==1) {pval <- " "}
      else if(length(level)>1 & by.order==FALSE) {pval <- myformat4(kruskal.test(var ~ by)$p.value)}
      else if(length(level)>1 & by.order==TRUE)  {pval <- myformat4(cor.test(var, as.numeric(by),alternative="two.sided",method="spearman")$p.value)}
      
      if(missing(note)){line1 <- paste(label,paste(textby,collapse=' & '),text,pval,sep=' & ')
      } else{line1 <- paste(paste(label,"\\\\tnote{",note,"}",sep=""),paste(textby,collapse=' & '),text,pval,sep=' & ')
      }
  } else{
      if(missing(note)){line1 <- paste(label,paste(textby,collapse=' & '),text,sep=' & ')
      } else{line1 <- paste(paste(label,"\\\\tnote{",note,"}",sep=""),paste(textby,collapse=' & '),text,sep=' & ')
      }
  }
  if(sum(is.na(var))>0){
    no <- table(is.na(var),by)[2,]
    nop <- sapply(100*no/table(by),myper1)
    nopt <- sapply(100*sum(no)/sum(table(by)),myper1)
    if(lower.label==TRUE){line2 <- paste(paste("\\\\hspace{.1in}Missing",ifelse(miss.label,paste(" ",tolower(label),sep=""),""),", n(\\\\%)",sep=""),paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),paste(sum(no)," (",nopt,"\\\\%)",sep=""),sep=" & ")}
      else{line2 <- paste(paste("\\\\hspace{.1in}Missing",ifelse(miss.label,paste(" ",label,sep=""),""),", n(\\\\%)",sep=""),paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),paste(sum(no)," (",nopt,"\\\\%)",sep=""),sep=" & ")}
    line3 <- paste(line1,line2,sep=" \\\\\\\\ ")
    return(list(line=line1,line.incmiss=line3))
  }
  else{return(list(line=line1))}
}
           
getrangetex <- function(var,by,label="",by.order=FALSE,digits=8,formatper="myper0",rangesep=" -- "){
  label <- ifelse(label=="",label(var),label)
  if(label==""){stop("label was not specified, either as an attribute of var, or in the getrangetex call.")}
  myper1 <- get(formatper)
  if(!is.numeric(var)){stop("The variable is non-numeric")}
  s <- round(summary(var),digits=digits)
  text <- paste(s[1],rangesep,s[6],sep="")
  level <- sort(unique(by))
  sby <- matrix(nrow=length(level), ncol=length(s)); textby <- rep(NA,length(level))
  for(i in 1:length(level)){
      sby[i,1:6] <- round(summary(var[by==level[i]])[1:6],digits=digits)
      textby[i] <- ifelse(!is.na(sby[i,3]),paste(sby[i,1],rangesep,sby[i,6],sep=""), "--")
  }
  line1 <- paste(paste0(label,", Range"),paste(textby,collapse=' & '),text,sep=' & ')
  return(list(line=line1))
}     

## PRINT N (%) FOR VAR WITHIN EACH BY COLUMN AND OVERALL
# options include 'by.order' and 'var.order' which will affect the hypothesis test
# note will include \tnote{note} by the label
# OUTPUT INCLUDES line and line.incmiss (if there is missing data)
getnptex <- function(var,by,label="",rowlabel=rowlabel,note=note,inc.pval=TRUE,var.order=FALSE,by.order=FALSE,formatper="myper0",rowper=FALSE){
  label <- ifelse(label=="",label(var),label)
  if(label==""){stop("label was not specified, either as an attribute of var, or in the getnptex call.")}
  myper1 <- get(formatper)
  if(missing(rowlabel)){
    if(is.factor(var)){rowlabel <- levels(var)}
    if(is.character(var)){rowlabel <- names(table(var))}
  }
  if(length(rowlabel) != sum(!(unique(var)%in% NA))){warning("Function GETNPTEX: number of unique values does not match length of rowlabel")}
  n <- table(var,by)
  n1 <- rowSums(n)
  d <- colSums(table(var,by)[1:nrow(n),])
  p <- matrix(nrow=nrow(n), ncol=ncol(n))
  pt <- rep(NA,nrow(n))
  lines <- rep(NA,nrow(n))
  for(i in 1:nrow(n)){
    p[i,which(d==0)] <- 0
    if(!rowper){p[i,which(d!=0)] <- sapply(100*n[i,which(d!=0)]/d[which(d!=0)],myper1)}
    if(rowper){p[i,which(d!=0)] <- sapply(100*n[i,which(d!=0)]/n1[i],myper1)}
    pt[i] <- sapply(100*table(var)[i]/sum(table(var)[1:nrow(n)]),myper1)
    lines[i] <- paste(paste("\\\\hspace{.2in}",rowlabel[i]),paste(paste(n[i,]," (",p[i,],"\\\\%)",sep=""),collapse=" & "),paste(sum(n[i,])," (",pt[i],"\\\\%)",sep=""),sep=" & ")
  }
# ADD PVALUE TO COLUMN HEADING IF DESIRED
  if(inc.pval==TRUE){
    if(by.order==TRUE){pval <- myformat4(kruskal.test(as.numeric(by), var)$p.value)}
    if(var.order==TRUE){pval <- myformat4(kruskal.test(var, by)$p.value)}
    if(by.order==TRUE & var.order==TRUE){pval <- myformat4(cor.test(var, by, alternative="two.sided", method="spearman")$p.value)} # ADDED 12/30/2010
    if(by.order==FALSE & var.order==FALSE){pval <- myformat4(chisq.test(n[n1!=0,d!=0])$p.value)}
    spaces <- rep(" ",1+length(table(by)))
    line1 <- paste(paste(label,", n(\\\\%)",sep=""),paste(spaces,collapse=" & "),pval,sep=" & ")
  } else{
    line1 <- paste(label,", n(\\\\%)",sep="")
  }
  all.lines <- paste(line1,paste(lines[which(n1!=0)],collapse=" \\\\\\\\ "),sep=" \\\\\\\\ ")
  if(sum(is.na(var))>0){
    no <- table(is.na(var),by)[2,]
    nop <- sapply(100*no/table(by),myper1)
    nopt <- sapply(100*sum(no)/sum(table(by)),myper1)
    if(missing(note)){ line2 <- paste("\\\\hspace{.1in}Missing",paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),paste(sum(no)," (",nopt,"\\\\%)",sep=""),sep=" & ")
    } else{line2 <- paste(paste("\\\\hspace{.1in}Missing\\\\tnote{",note,"}",sep=""),paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),paste(sum(no)," (",nopt,"\\\\%)",sep=""),sep=" & ")
    }
    all.lines.miss <- paste(line1,line2,paste(lines,collapse=" \\\\\\\\ "),sep=" \\\\\\\\ ")
    return(list(line=all.lines,line.incmiss=all.lines.miss))
  }
  else{return(list(line=all.lines))}
}

simpleCap <- function(s) {
         paste(toupper(substring(s, 1,1)), tolower(substring(s, 2)),
               sep="", collapse=" ")
     }


# find N, percent by some variable and overall
getnp <- function(var,by){
  n <- table(var,by)
  d <- colSums(table(var,by)[1:nrow(n),])
  p <- matrix(nrow=nrow(n), ncol=ncol(n))
  pt <- rep(NA,nrow(n))
  pval <- myformat3(chisq.test(n)$p.value)

  for(i in 1:nrow(n)){
    p[i,] <- sapply(100*n[i,]/d,myper1)
    pt[i] <- sapply(100*table(var)[i]/sum(table(var)[1:nrow(n)]),myper1)
  }
  if(sum(is.na(var))>0){
    no <- table(is.na(var),by)[2,]
    nop<- round(no/table(by),3)*100   
    nopt <- round(sum(no)/sum(table(by)),3)*100   
    return(list(n=n,p=p,pt=pt,pval=pval,no=no,nop=nop,nopt=nopt))
  }
  else{return(list(n=n,p=p,pt=pt,pval=pval))}
}

# find median (Q1,Q3) missing N, percent by some variable and overall
getmed <- function(var,by,iqrsep="--"){
  if(!is.numeric(var)){stop("The variable is non-numeric")}
  s <- round(summary(var),2)
  text <- paste(s[3]," (",s[2],iqrsep,s[5],")",sep="")
  level <- sort(unique(by))
  sby <- matrix(nrow=length(level), ncol=length(s))
  textby <- rep(NA,length(level))
  if(length(level)==1) {}
  else if(length(level)==2){pval <- myformat3(wilcox.test(var ~ by)$p.value)}
  else if(length(level)>2) {pval <- myformat3(kruskal.test(var ~ by)$p.value)}
  for(i in 1:length(level)){
      sby[i,1:5] <- round(summary(var[by==level[i]])[1:5],2)
      textby[i] <- paste(sby[i,3]," (",sby[i,2],iqrsep,sby[i,5],")",sep="")    
  }
  if(sum(is.na(var))>0){
    no <- table(is.na(var),by)[2,]
    nop<- round(no/table(by),3)*100   
    nopt <- round(sum(no)/sum(table(by)),3)*100   
    return(list(text=text,textby=textby,pval=pval,no=no,nop=nop,nopt=nopt))
  }
  else{return(list(text=text,textby=textby,pval=pval))}
}


# LONGITUDINAL VERSION OF GETMEDTEX (developed for NEMART)
# when by is a timepoint like visit #, then we do not want a total column
# additionally we need to change the test of significance
# MUST RUN library(nlme)
longmedtex <- function(var,by,caseid,label,note=note,inc.pval=TRUE,lower.label=TRUE,formatper="myper0",iqrsep="--"){
  myper1 <- get(formatper)
  s <- round(summary(var),1)
  level <- sort(unique(by))
  sby <- matrix(nrow=length(level), ncol=length(s)); textby <- rep(NA,length(level))
  if(length(level)==1) {pval <- " "}
#  else if(length(level)>1) {
#      model1 <- lmer(var ~ by + (by|caseid))
#      model2 <- lmer(var ~ by + (1|caseid) + (0+by|caseid))
#      pval <- myformat3(anova(model1,model2)$'Pr(>Chisq)'[2])
#  }
  for(i in 1:length(level)){
      sby[i,1:5] <- round(summary(var[by==level[i]])[1:5],1)
      if(!is.na(sby[i,3])){textby[i] <- paste(sby[i,3],iqrsep,sby[i,2],iqrsep,sby[i,5],")",sep="")}
      else if(is.na(sby[i,3])){textby[i] <- "-"}
  }
  if(inc.pval==TRUE & length(level)>1){
      newdata1 <- data.frame(var,by,caseid)[!is.na(var),]
      newdata <- groupedData(var ~ by | caseid, data=newdata1)
      mylme <- lme(var ~ by, data=newdata, random=~1|caseid, na.action=na.omit,method="ML")
      pval <- myformat4(as.data.frame(anova(mylme))$'p-value'[2])
      if(missing(note)){line1 <- paste(label,paste(textby,collapse=' & '),pval,sep=' & ')
      } else{line1 <- paste(paste(label,"\\\\tnote{",note,"}",sep=""),paste(textby,collapse=' & '),pval,sep=' & ')
      }
  } else{
      if(missing(note)){line1 <- paste(label,paste(textby,collapse=' & '),sep=' & ')
      } else{line1 <- paste(paste(label,"\\\\tnote{",note,"}",sep=""),paste(textby,collapse=' & '),sep=' & ')
      }
  }
  if(sum(is.na(var))>0){
    no <- table(is.na(var),by)[2,]
    nop <- sapply(100*no/table(by),myper1)
    if(lower.label==TRUE){line2 <- paste(paste("\\\\hspace{.1in}Missing ",tolower(label),", n(\\\\%)",sep=""),paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),sep=" & ")}
      else{line2 <- paste(paste("\\\\hspace{.1in}Missing ",label,", n(\\\\%)",sep=""),paste(paste(no," (",nop,"\\\\%)",sep=""),collapse=" & "),sep=" & ")}
    line3 <- paste(line1,line2,sep=" \\\\\\\\ ")
    return(list(line=line1,line.incmiss=line3))
  }
  else{return(list(line=line1))}
}


######################
## ADDED 2011-06-20 ##
######################
## GETNPUNI AND GETMEDUNI basically produce the 'combined' only column in case I do not have a variable of interest for crosstabulation - just drop 'by'
getmeduni <- function(var,label="",note=note,lower.label=TRUE,digits=8,formatper="myper0",iqrsep=" -- "){
  label <- ifelse(label=="",label(var),label)
  if(label==""){stop("label was not specified, either as an attribute of var, or in the getmeduni call.")}
  myper1 <- get(formatper)
  if(!is.numeric(var)){stop("The variable is non-numeric")}
  s <- round(summary(var),digits=digits)
  text <- paste(s[3]," (",s[2],iqrsep,s[5],")",sep="")

  if(missing(note)){line1 <- paste(label,text,sep=' & ')
      } 
  else{line1 <- paste(paste(label,"\\\\tnote{",note,"}",sep=""),text,sep=' & ')
      }
  if(sum(is.na(var))>0){
    no <- sum(is.na(var))
    nop <- sapply(100*no/length(var),myper1)
    if(lower.label==TRUE){
	line2 <- paste(paste("\\\\hspace{.1in}Missing ",tolower(label),", n(\\\\%)",sep=""),paste(no," (",nop,"\\\\%)",sep=""),sep=" & ")
    }
    else{
	line2 <- paste(paste("\\\\hspace{.1in}Missing ",label,", n(\\\\%)",sep=""),paste(no," (",nop,"\\\\%)",sep=""),sep=" & ")
    }
    line3 <- paste(line1,line2,sep=" \\\\\\\\ ")
    return(list(line=line1,line.incmiss=line3))
  }
  else{return(list(line=line1))}
}

getnpuni <- function(var,label="",rowlabel=rowlabel,note=note,formatper="myper0"){
  label <- ifelse(label=="",label(var),label)
  if(label==""){stop("label was not specified, either as an attribute of var, or in the getnpuni call.")}
  myper1 <- get(formatper)
  if(missing(rowlabel)){
    if(is.factor(var)){rowlabel <- levels(var)}
    if(is.character(var)){rowlabel <- names(table(var))}
  }
  if(length(rowlabel) != sum(!(unique(var)%in% NA))){warning("Function GETNPUNI: number of unique values does not match length of rowlabel")}
  n <- table(var)
  p <- sapply(100*n/sum(n),myper1)

  lines <- paste(paste("\\\\hspace{.2in}",rowlabel),paste(n," (",p,"\\\\%)",sep=""),sep=" & ")
  line1 <- paste(label,", n(\\\\%)",sep="")

  all.lines <- paste(line1,paste(lines[which(n!=0)],collapse=" \\\\\\\\ "),sep=" \\\\\\\\ ")
  if(sum(is.na(var))>0){
    no <- sum(is.na(var))
    nop <- sapply(100*no/length(var),myper1)
    if(missing(note)){ 
      line2 <- paste("\\\\hspace{.1in}Missing",paste(no," (",nop,"\\\\%)",sep=""),sep=" & ")
    } else{
      line2 <- paste(paste("\\\\hspace{.1in}Missing\\\\tnote{",note,"}",sep=""),paste(no," (",nop,"\\\\%)",sep=""),sep=" & ")
    }
    all.lines.miss <- paste(line1,line2,paste(lines,collapse=" \\\\\\\\ "),sep=" \\\\\\\\ ")
    return(list(line=all.lines,line.incmiss=all.lines.miss))
  }
  else{return(list(line=all.lines))}
}

getnp1lineuni <- function(var,rowlabel="",indent=TRUE,formatper="myper0"){
  rowlabel <- ifelse(rowlabel=="",label(var),rowlabel)
  if(rowlabel==""){stop("rowlabel was not specified, either as an attribute of var, or in the getnp1lineuni call.")}
  myper1 <- get(formatper)
  n <- table(var)
  p <- sapply(100*n/length(var),myper1)
  if(indent==TRUE){
      line <- paste(paste("\\\\hspace{.2in}",rowlabel),paste(n," (",p,"\\\\%)",sep=""),sep=" & ")
  }
  else{
      line <- paste(paste(rowlabel),paste(n," (",p,"\\\\%)",sep=""),sep=" & ")
  }
  return(list(line=line))
}



## THIS IS FROM JENNIFER'S 9/25 EMAIL

## -- Export data from REDCap -- ##
get.api.data <- function(use.token){
  curl_handle = getCurlHandle()
  curlSetOpt(ssl.verifypeer = FALSE, connecttimeout = 0, timeout = 600, curl = curl_handle)
  
  api.data <- postForm('https://redcap.vanderbilt.edu/api/',
                       token = use.token,
                       content = 'record',
                       format = 'csv',
                       type = 'flat',
                       exportDataAccessGroups = TRUE)
  data <- read.csv(file = textConnection(api.data),
                   header = TRUE,
                   sep = ',',
                   na.strings = '',
                   stringsAsFactors = FALSE)
  
  return(data)
}

## THIS IS JOANN's FUNCTION
##  http://biostat.mc.vanderbilt.edu/wiki/pub/Main/JoAnnAlvarez/redcapFun.R


redcapFun <- function(APIKEY) {
    library(RCurl)
    rcon <- redcapConnection('https://redcap.vanderbilt.edu/api/', APIKEY)
    gg <- exportMetaData(rcon)

    # reverse a string
    strReverse <- function(x) sapply(lapply(strsplit(x, NULL), rev), paste, collapse='')
    # remove quotes
    rmq <- function(x) gsub("['\"]", '', x)
    clean <- function(x) { gsub('[\n]', ' ', rmq(x)) }

    labels <- vector('list', nrow(gg))
    factors <- vector('list', nrow(gg))
    levels <- vector('list', nrow(gg))
    # remove quote characters, replace line breaks
    gg$field_label <- gsub('[\x80-\x9f]', '', clean(gg$field_label), useBytes=TRUE)

    for(i in seq(nrow(gg))) {
        fld <- gg[i,]
        label <- NULL
        factor <- NULL
        level <- NULL
        if(fld$field_type == 'checkbox') {
            choices <- strsplit(fld$select_choices_or_calculations, '[|]')[[1]]
            choices <- sub('^[ ]+(.*)$', '\\1', choices)
            choices <- sub('(.*)[ ]+$', '\\1', choices)
            nums <- sub('^([^,]+).*$', '\\1', choices)
            choices <- sub('^[^,]+[, ]+(.*)$', '\\1', choices)
            label <- sprintf('label(data$%s___%s) <- "%s (choice=%s)"', fld$field_name, nums, sub('[\n]', ' ', fld$field_label), rmq(choices))
            factor <- sprintf('data$%1$s___%2$s <- factor(data$%1$s___%2$s, levels=c("0", "1"))', fld$field_name, nums)
            level <- sprintf('levels(data$%s___%s) <- c("Unchecked", "Checked")', fld$field_name, nums)
        } else if(fld$field_type %in% c('radio', 'dropdown')) {
            label <- sprintf('label(data$%s) <- "%s"', fld$field_name, sub('[\n]', ' ', fld$field_label))
            factor <- sprintf('attr(data$%s, "redcapLevels") <- NULL', fld$field_name)
        } else if(fld$field_type == 'yesno') {
            label <- sprintf('label(data$%s) <- "%s"', fld$field_name, sub('[\n]', ' ', fld$field_label))
            factor <- sprintf('data$%1$s <- factor(data$%1$s, levels=c("1", "0"))', fld$field_name)
            level <- sprintf('levels(data$%s) <- c("Yes", "No")', fld$field_name)
        } else if(fld$field_type == 'truefalse') {
            label <- sprintf('label(data$%s) <- "%s"', fld$field_name, sub('[\n]', ' ', fld$field_label))
            factor <- sprintf('data$%1$s <- factor(data$%1$s, levels=c("1", "0"))', fld$field_name)
            level <- sprintf('levels(data$%s) <- c("True", "False")', fld$field_name)
        } else if(fld$field_type %in% c('notes', 'text', 'slider', 'sql')) {
            label <- sprintf('label(data$%s) <- "%s"', fld$field_name, sub('[\n]', ' ', fld$field_label))
        } else {
            print(sprintf("I don't know field_type %s for field %s", fld$field_type, fld$field_name))
        }
        labels[[i]] <- label
        factors[[i]] <- factor
        levels[[i]] <- level
    }

    # <instrument>_complete
    complete <- sprintf('%s_complete', unique(gg$form_name))
    labels[[nrow(gg)+1]] <- sprintf('label(data$%s) <- "Complete?"', complete)
    factors[[nrow(gg)+1]] <- sprintf('data$%1$s <- factor(data$%1$s, levels=c("2", "1", "0"))', complete)
    levels[[nrow(gg)+1]] <- sprintf('levels(data$%s) <- c("Complete", "Unverified", "Incomplete")', complete)

    info <- c('library(Hmisc)', unlist(factors), unlist(levels), unlist(labels))
    # write(info, 'data_labels.r')
    data <- get.api.data(APIKEY)
    # old call using Will's function.... data <- exportRecords(rcon)
    # source('data_labels.r')
    eval(parse(text=info))
    # data <- as.data.frame(lapply(data, ))
    return(data)
}





### FROM: http://stackoverflow.com/questions/3402371/rbind-different-number-of-columns
### combines data frames (like rbind) but by matching column names
# columns without matches in the other data frame are still combined
# but with NA in the rows corresponding to the data frame without
# the variable
# A warning is issued if there is a type mismatch between columns of
# the same name and an attempt is made to combine the columns
combineByName <- function(A,B) {
  a.names <- names(A)
  b.names <- names(B)
  all.names <- union(a.names,b.names)
  print(paste("Number of columns:",length(all.names)))
  a.type <- NULL
  for (i in 1:ncol(A)) {
    a.type[i] <- typeof(A[,i])
  }
  b.type <- NULL
  for (i in 1:ncol(B)) {
    b.type[i] <- typeof(B[,i])
  }
  a_b.names <- names(A)[!names(A)%in%names(B)]
  b_a.names <- names(B)[!names(B)%in%names(A)]
  if (length(a_b.names)>0 | length(b_a.names)>0){
    print("Columns in data frame A but not in data frame B:")
    print(a_b.names)
    print("Columns in data frame B but not in data frame A:")
    print(b_a.names)
  } else if(a.names==b.names & a.type==b.type){
    C <- rbind(A,B)
    return(C)
  }
  C <- list()
  for(i in 1:length(all.names)) {
    l.a <- all.names[i]%in%a.names
    pos.a <- match(all.names[i],a.names)
    typ.a <- a.type[pos.a]
    l.b <- all.names[i]%in%b.names
    pos.b <- match(all.names[i],b.names)
    typ.b <- b.type[pos.b]
    if(l.a & l.b) {
      if(typ.a==typ.b) {
        vec <- c(A[,pos.a],B[,pos.b])
      } else {
        warning(c("Type mismatch in variable named: ",all.names[i],"\n"))
        vec <- try(c(A[,pos.a],B[,pos.b]))
      }
    } else if (l.a) {
      vec <- c(A[,pos.a],rep(NA,nrow(B)))
    } else {
      vec <- c(rep(NA,nrow(A)),B[,pos.b])
    }
    C[[i]] <- vec
  }
  names(C) <- all.names
  C <- as.data.frame(C,stringsAsFactors=TRUE)
  return(C)
}
